define bi_yum::repo (
    $repositoryid = $title,
    $descr = undef,
    $mirrorlist = undef,
    $baseurl = undef,
    $failovermethod = undef,
    $enabled = 1,
    $gpgcheck = 1,
    $gpgkey = undef,
    $gpgkey_url = undef,
) {
    if !$mirrorlist and !$baseurl { 
        fail("'mirrorlist' or 'baseurl' must be defined") 
    }
    
    if !$repositoryid { 
        fail("'repositoryid' must be defined") 
    }
    
    if !$gpgkey { 
        fail("'gpgkey' must be defined") 
    }
    
    if !$enabled { 
        fail("'enabled' must be defined") 
    }
    
    if !$gpgcheck { 
        fail("'gpgcheck' must be defined") 
    }
    
    require ::bi_wget
    require ::bi_coreutils

    $repofilename = "bi-$repositoryid"
    $pgpfilename = regsubst($gpgkey, 'file://', '')
    
    yumrepo { $repositoryid:
        target => $repofilename,
        descr => $descr,
        mirrorlist => $mirrorlist,
        baseurl => $baseurl,
        failovermethod => $failovermethod,
        enabled => $enabled,
        gpgcheck => $gpgcheck,
        gpgkey => $gpgkey,
    }
    
    case $gpgkey_url {
        /^puppet:\/\//: {
            file { $pgpfilename:
                owner => 'root',
                group => 'root',
                mode => '644',
                source => $gpgkey_url,
            }
        }
        
        /^http(s?):\/\//: {          
            bi_wget::wget { $gpgkey_url:
                output_document => $pgpfilename,
                no_clobber => true,
            }
            
            bi_coreutils::chmod { $pgpfilename:
                mode => '644',
                require => Bi_wget::Wget[$gpgkey_url],
            }
            
            bi_coreutils::chown { $pgpfilename:
                owner => 'root',
                group => 'root',
                require => Bi_wget::Wget[$gpgkey_url],
            }
        }
        
        default: { fail("unsupported protocol: '$gpgkey_url'") }
    }
}